# Gosl. gm/msh. Mesh generation and Delaunay triangulation

[![PkgGoDev](https://pkg.go.dev/badge/github.com/cpmech/gosl/gm/msh)](https://pkg.go.dev/github.com/cpmech/gosl/gm/msh)

## Definitions

![](data/meshelems1.png)

### Quadrilaterals and Hexahedra

```
  2D:
                Vertices                              Edges
   y
   |        3      6      2                             2
   +--x      @-----@-----@                        +-----------+
             |           |                        |           |
             |           |                        |           |
           7 @           @ 5                     3|           |1
             |           |                        |           |
             |           |                        |           |
             @-----@-----@                        +-----------+
            0      4      1                             0

  3D:
                  Vertices                            Faces
    z
    |           4        15        7
   ,+--y         @-------@--------@                 +----------------+
 x'            ,'|              ,'|               ,'|              ,'|
          12 @'  |         14 ,'  |             ,'  |  ___       ,'  |
           ,'    |16        ,@    |19         ,'    |,'5,'  [0],'    |
     5   ,'      @      6 ,'      @         ,'      |~~~     ,'      |
       @'=======@=======@'        |       +'===============+'  ,'|   |
       |      13 |      |         |       |   ,'|   |      |   |3|   |
       |         |      |  11     |       |   |2|   |      |   |,'   |
    17 |       0 @- - - | @- - - -@       |   |,'   +- - - | +- - - -+
       @       ,'       @       ,' 3      |       ,'       |       ,'
       |   8 @'      18 |     ,'          |     ,' [1]  ___|     ,'
       |   ,'           |   ,@ 10         |   ,'      ,'4,'|   ,'
       | ,'             | ,'              | ,'        ~~~  | ,'
       @-------@--------@'                +----------------+'
     1         9         2

  3D:
                  Seams (aka Edges)
    z
    |                    7
   ,+--y         +----------------+
 x'            ,'|              ,'|
           4 ,'  |8          6,'  |
           ,'    |          ,'    |
         ,'      |   5    ,'      |11
       +'===============+'        |
       |         |      |         |
       |         |      |  3      |
       |         .- - - | -  - - -+
      9|       ,'       |       ,'
       |    0,'         |10   ,'
       |   ,'           |   ,' 2
       | ,'             | ,'
       +----------------+'
               1
```

### Triangles and Tetrahedra

```
  2D:
             Nodes                      Edges

   y           2
   |           @                          @
   +--x       / \                        / \
           5 /   \ 4                    /   \
            @     @                  2 /     \ 1
           /       \                  /       \
          /         \                /         \
         @-----@-----@              @-----------@
        0      3      1                   0


  3D:              Nodes                                     Faces
    z
    |                |                                          |
   ,+--y             3                                          +
 x'                 /|`.                                       /|`.
                    ||  `,                                     ||  `,
                   / |    ',                                  / |    ',
                   | |      \                                 | |      \
                  /  |       `.                              /  |       `.
                  |  |         `,                            |  |         `,
                 /   7            9                         /   |           ',
                 |   |             \                        |   |             \
                /    |              `.                     /    |     0        `.
                |    |                ',                   |    |                ',
               8     |                  \                 /  1  |                  \
               |     0 ,,_               `.               |     + ,,_               `.
              |     /     ``'-., 6         `.            |     /     ``'-.,,.         `.
              |    /               `''-.,,_  ',          |    /              ``''-.,,_  ',
             |    /                        ``'2 ,,      |    /      \3\               ``'+ ,,
             |   '                       ,.-``          |  ,'                       ,.-``
            |   4                   _,-'`              |  /                    _,-'`
            ' /                 ,.'`                   ' /      2          ,.'`
           | /             _ 5 `                      | /             _.''`
           '/          ,-'`                           '/          ,-'`
          |/      ,.-``                              |/      ,.-``
          /  _,-``                                   /  _,-``
         1 '`                                       +.'`

    3D:
         Seams (aka edges)

               |
               +
              /|`.
              ||  `,
             / |    ',
             | |      \
            /  |       `.
            |  |         `,5
           /   |3          ',
           |   |             \
          /    |              `.
        4 |    |                ',
         /     |                  \
         |     +.,,_               `.
        |     /     ``'-.,,.         `.
        |    /          2   ``''-.,,_  ',
       |    /                        ``'+ ,,
       |  ,'                       ,.-``
      |  / 0                  _,-'`
      ' /                 ,.'`
     | /             _.''`
     '/          ,-'`   1
    |/      ,.-``
    /  _,-``
   +.'`
```

## API

[Please see the documentation here](https://pkg.go.dev/github.com/cpmech/gosl/gm/msh)
